const request = require('superagent');
const now = require('nano-time');

/**
 * Store data to influxdb
 * @param {Object} data params
 * @param {string} data.url link to http endpoint
 * @param {string} data.db database name
 * @param {string} data.meas measurement name
 * @param {Object} data.tags tags to add
 * @param {Object} data.values values to send
 * @param {number} data.timestamp time in nanoseconds
 * @param {number} data.retry superagent retries
 * @param {Object|number} data.timeout superagent timeout
 */
const writeToInflux = async ({
    url, db, meas, tags, values,
    timestamp = now(),
    retry = 3,
    timeout = {response: 10000, deadline: 20000},
}) => {
    const sendTags = Object.entries(tags).map(elem => elem.join('=')).join();
    const sendData = Object.entries(values).map(elem => {
        const [, prop] = elem;
        elem[1] = typeof prop === 'string' ? `"${prop}"` : prop;
        return elem.join('=');
    }).join();

    try {
        await request
            .post(`${url}/write`)
            .query({db})
            .send(`${meas},${sendTags} ${sendData} ${timestamp}`)
            .retry(retry)
            .timeout(timeout);

    } catch (err) {
        throw new Error([
            'Error sending data to influx:',
            `URL: ${url}`,
            `DB: ${db}`,
            `Measurement: ${meas}`,
            `Tags: ${sendTags}`,
            `Values: ${sendData}`,
            `Timestamp: ${timestamp}`,
            err,
        ].join('\n'));
    }
};

module.exports = writeToInflux;
